<?php
declare(strict_types=1);

namespace CakeProducts\Controller;

use Cake\Core\Configure;
use Cake\Datasource\Exception\RecordNotFoundException;
use Cake\Http\Response;
use Cake\Log\Log;
use CakeProducts\Controller\AppController;
use CakeProducts\Model\Table\ProductCatalogsTable;
use CakeProducts\Service\CatalogManagerService;

/**
 * ProductCatalogs Controller
 *
 * @property ProductCatalogsTable $ProductCatalogs
 */
class ProductCatalogsController extends AppController
{
    /**
     * Index method
     *
     * @return Response|null|void Renders view
     */
    public function index()
    {
        $query = $this->ProductCatalogs->find();
        $productCatalogs = $this->paginate($query);

        $this->set(compact('productCatalogs'));
    }

    /**
     * View method
     *
     * @param string|null $id Product Catalog id.
     * @return Response|null|void Renders view
     * @throws RecordNotFoundException When record not found.
     */
    public function view($id = null)
    {
        $contain = ['ProductCategories'];
        if (Configure::read('CakeProducts.internal.syncExternally', false)) {
            $contain[] = 'ExternalProductCatalogs';
        }
        $productCatalog = $this->ProductCatalogs->get($id, contain: $contain);
        $this->set(compact('productCatalog'));
    }

    /**
     * Add method
     *
     * @return Response|null|void Redirects on successful add, renders view otherwise.
     */
    public function add()
    {
        $productCatalog = $this->ProductCatalogs->newEmptyEntity();
        if ($this->request->is('post')) {
            $productCatalog = $this->ProductCatalogs->patchEntity($productCatalog, $this->request->getData());
            if ($this->ProductCatalogs->save($productCatalog)) {
                $this->Flash->success(__('The product catalog has been saved.'));

                return $this->redirect(['action' => 'index']);
            }
            Log::debug('failed to save new product catalog errors next');
            Log::debug(print_r('$productCatalog->getErrors()', true));
            Log::debug(print_r($productCatalog->getErrors(), true));

            $this->Flash->error(__('The product catalog could not be saved. Please, try again.'));
        }
        $this->set(compact('productCatalog'));
    }

    /**
     * Edit method
     *
     * @param string|null $id Product Catalog id.
     * @return Response|null|void Redirects on successful edit, renders view otherwise.
     * @throws RecordNotFoundException When record not found.
     */
    public function edit($id = null)
    {
        $productCatalog = $this->ProductCatalogs->get($id, contain: []);
        if ($this->request->is(['patch', 'post', 'put'])) {
            $productCatalog = $this->ProductCatalogs->patchEntity($productCatalog, $this->request->getData());
            if ($this->ProductCatalogs->save($productCatalog)) {
                $this->Flash->success(__('The product catalog has been saved.'));

                return $this->redirect(['action' => 'index']);
            }
            $this->Flash->error(__('The product catalog could not be saved. Please, try again.'));
        }
        $this->set(compact('productCatalog'));
    }

    /**
     * Delete method
     *
     * @param string|null $id Product Catalog id.
     * @return Response|null Redirects to index.
     * @throws RecordNotFoundException When record not found.
     */
    public function delete($id = null)
    {
        $this->request->allowMethod(['post', 'delete']);
        $productCatalog = $this->ProductCatalogs->get($id);
        if ($this->ProductCatalogs->delete($productCatalog)) {
            $this->Flash->success(__('The product catalog has been deleted.'));
        } else {
            $this->Flash->error(__('The product catalog could not be deleted. Please, try again.'));
        }

        return $this->redirect(['action' => 'index']);
    }
}
